@@ -28,7 +28,7 @@ Console::Console(){
 	maxLines= Config::getInstance().getInt("ConsoleMaxLines");
 	timeout= Config::getInstance().getInt("ConsoleTimeout");
 
-	timeElapsed= timeout;
+	timeElapsed= 0.0f;
 }
 
 void Console::addStdMessage(const string &s){
@@ -36,18 +36,17 @@ void Console::addStdMessage(const string &s){
 }
 
 void Console::addLine(string line){
-	timeElapsed= timeout;
-	lines.push_front(line);
+	lines.insert(lines.begin(), StringTimePair(line, timeElapsed));
 	if(lines.size()>maxLines){
 		lines.pop_back();
 	}
 }
 
 void Console::update(){
-	timeElapsed-= 1.f/GameConstants::updateFps;
-    if(timeElapsed<0){
-        timeElapsed= timeout;
-		if(!lines.empty()){
+	timeElapsed+= 1.f/GameConstants::updateFps;
+	
+	if(!lines.empty()){
+		if(lines.back().second<timeElapsed-timeout){
 			lines.pop_back();
 		}
     }   
