@@ -12,11 +12,13 @@
 #ifndef _GLEST_GAME_CONSOLE_H_
 #define _GLEST_GAME_CONSOLE_H_
 
+#include <utility>
 #include <string>
-#include <list>
+#include <vector>
 
 using std::string;
-using std::list;
+using std::vector;
+using std::pair;
 
 namespace Glest{ namespace Game{
 
@@ -31,12 +33,13 @@ private:
 	static const int consoleLines= 5;
 
 public:
-	typedef list<string> LineContainer;
-	typedef LineContainer::const_iterator LineIterator;
+	typedef pair<string, float> StringTimePair;
+	typedef vector<StringTimePair> Lines;
+	typedef Lines::const_iterator LineIterator;
 
 private:
 	float timeElapsed; 
-	LineContainer lines;
+	Lines lines;
 
 	//this should be deleted from here someday
 	bool won, lost;
@@ -48,8 +51,9 @@ private:
 public:
 	Console();
 	
-	LineIterator getLineBegin() const	{return lines.begin();}
-	LineIterator getLineEnd() const		{return lines.end();}
+	int getLineCount() const		{return lines.size();}
+	string getLine(int i) const		{return lines[i].first;}
+
 
 	void addStdMessage(const string &s);
 	void addLine(string line);
