@@ -27,6 +27,7 @@
 
 using namespace Shared::Graphics;
 using namespace Shared::Util;
+using namespace Shared::Platform;
 
 namespace Glest{ namespace Game{
 
@@ -52,6 +53,7 @@ Game::Game(Program *program, const GameSettings *gameSettings):
 	lastRenderFps=0;
 	paused= false;
 	gameOver= false;
+	renderNetworkStatus= false;
 	speed= sNormal;
 	exitMessageBox= NULL;
 }
@@ -98,10 +100,16 @@ void Game::init(){
 
 	logger.setState(Lang::getInstance().get("Initializing"));
 	
+	//check forog war
+	if(!Config::getInstance().getBool("FogOfWar") && networkManager.isNetworkGame() ){
+		throw runtime_error("Can not play online games with for of war disabled");
+	}
+
 	//init world, and place camera
 	commander.init(&world);
 	world.init(this);
 	gui.init(this);
+	chatManager.init(&console, world.getThisTeamIndex());
 	const Vec2i &v= map->getStartLocation(world.getThisFaction()->getStartLocationIndex());
 	gameCamera.init(map->getW(), map->getH());
 	gameCamera.setPos(Vec2f(v.x, v.y));
@@ -159,7 +167,7 @@ void Game::init(){
 	}
 
 	logger.add("Waiting for network", true);
-	networkManager.waitUntilReady(&checksum);
+	networkManager.getGameNetworkInterface()->waitUntilReady(&checksum);
 
 	logger.add("Starting music stream", true);
 	StrSound *gameMusic= world.getThisFaction()->getType()->getMusic();
@@ -173,8 +181,7 @@ void Game::init(){
 
 //update
 void Game::update(){
-	NetworkManager &networkManager= NetworkManager::getInstance();
-
+	
 	// a) Updates non dependant on speed
 
 	//misc
@@ -203,7 +210,7 @@ void Game::update(){
 		world.update();
 
 		// Commander
-		commander.update();
+		commander.updateNetwork();
 
 		//Gui
 		gui.update();
@@ -215,16 +222,12 @@ void Game::update(){
 		renderer.updateParticleManager(rsGame);
 	}
 
-	//listen to quit messages from the server
-	if(networkManager.getNetworkRole()==nrClient){
-		ClientInterface *clientInterface= networkManager.getClientInterface();
+	//call the chat manager
+	chatManager.updateNetwork();
 
-		if(clientInterface->getNextMessageType()==nmtQuit){
-			NetworkMessageQuit networkMessageQuit;
-			if(clientInterface->receiveMessage(&networkMessageQuit)){
-				program->setState(new BattleEnd(program, world.getStats()));
-			}
-		}
+	//check for quiting status
+	if(NetworkManager::getInstance().getGameNetworkInterface()->getQuit()){
+		program->setState(new BattleEnd(program, world.getStats()));
 	}
 }
 
@@ -302,10 +305,7 @@ void Game::mouseDownLeft(int x, int y){
 		int button= 1;
 		if(exitMessageBox->mouseClick(x, y, button)){
 			if(button==1){
-				//if we are the server, send messages to the clients
-				if(networkManager.getNetworkRole()==nrServer){
-					networkManager.getServerInterface()->quitGame();
-				}
+				networkManager.getGameNetworkInterface()->quitGame();
 				program->setState(new BattleEnd(program, world.getStats()));
 			}
 			else{
@@ -390,128 +390,161 @@ void Game::keyDown(char key){
 	Lang &lang= Lang::getInstance();
 	bool speedChangesAllowed= !NetworkManager::getInstance().isNetworkGame();
 
-	//move camera left
-	if(key==vkLeft){
-        gameCamera.setMoveX(-1);
-	}
+	//send ley to the chat manager
+	chatManager.keyDown(key);
 
-	//move camera right
-	else if(key==vkRight){
-        gameCamera.setMoveX(1);
-	}
+	if(!chatManager.getEditEnabled()){
 
-	//move camera up
-	else if(key==vkUp){
-        gameCamera.setMoveZ(1);
-	}
-
-	//move camera down
-	else if(key==vkDown){
-        gameCamera.setMoveZ(-1);
-	}
-
-	//switch display color
-	else if(key=='C'){
-		gui.switchToNextDisplayColor();
-	}
-
-	//change camera mode
-	else if(key=='F'){
-		gameCamera.switchState();
-		string stateString= gameCamera.getState()==GameCamera::sGame? lang.get("GameCamera"): lang.get("FreeCamera");
-		console.addLine(lang.get("CameraModeSet")+" "+ stateString);
-	}
-
-	//pause
-	else if(key=='P'){
-		if(speedChangesAllowed){
-			if(paused){
-				console.addLine(lang.get("GameResumed"));
-				paused= false;
-			}
-			else{
-				console.addLine(lang.get("GamePaused"));
-				paused= true;
+		if(key=='N'){
+			renderNetworkStatus= true;
+		}
+		else if(key=='E'){
+			for(int i=0; i<100; ++i){
+				string path= "screens/screen" + intToStr(i) + ".tga";
+				
+				FILE *f= fopen(path.c_str(), "rb");
+				if(f==NULL){
+					Renderer::getInstance().saveScreen(path);
+					break;
+				}
+				else{
+					fclose(f);
+				}
 			}
 		}
-	}
 
-	//increment speed
-	else if(key==vkAdd){
-		if(speedChangesAllowed){
-			incSpeed();
-		}
-	}
-
-	//decrement speed
-	else if(key==vkSubtract){
-		if(speedChangesAllowed){
-			decSpeed();
-		}
-	}
-
-	//exit
-	else if(key==vkEscape){
-		showExitMessageBox(lang.get("ExitGame?"), true);
-	}
-
-	//group
-	else if(key>='0' && key<'0'+Selection::maxGroups){
-		gui.groupKey(key-'0');
-	}
-
-	//hotkeys
-	if(gameCamera.getState()==GameCamera::sGame){
-		gui.hotKey(key);
-	}
-	else{
-		//rotate camera leftt
-		if(key=='A'){
-			gameCamera.setRotate(-1);
+		//move camera left
+		else if(key==vkLeft){
+			gameCamera.setMoveX(-1);
 		}
 
-		//rotate camera right
-		else if(key=='D'){
-			gameCamera.setRotate(1);
+		//move camera right
+		else if(key==vkRight){
+			gameCamera.setMoveX(1);
 		}
 
-		//camera up
-		else if(key=='S'){
-			gameCamera.setMoveY(1);
+		//move camera up
+		else if(key==vkUp){
+			gameCamera.setMoveZ(1);
 		}
 
-		//camera down
-		else if(key=='W'){
-			gameCamera.setMoveY(-1);
+		//move camera down
+		else if(key==vkDown){
+			gameCamera.setMoveZ(-1);
+		}
+
+		//switch display color
+		else if(key=='C'){
+			gui.switchToNextDisplayColor();
+		}
+
+		//change camera mode
+		else if(key=='F'){
+			gameCamera.switchState();
+			string stateString= gameCamera.getState()==GameCamera::sGame? lang.get("GameCamera"): lang.get("FreeCamera");
+			console.addLine(lang.get("CameraModeSet")+" "+ stateString);
+		}
+
+		//pause
+		else if(key=='P'){
+			if(speedChangesAllowed){
+				if(paused){
+					console.addLine(lang.get("GameResumed"));
+					paused= false;
+				}
+				else{
+					console.addLine(lang.get("GamePaused"));
+					paused= true;
+				}
+			}
+		}
+
+		//increment speed
+		else if(key==vkAdd){
+			if(speedChangesAllowed){
+				incSpeed();
+			}
+		}
+
+		//decrement speed
+		else if(key==vkSubtract){
+			if(speedChangesAllowed){
+				decSpeed();
+			}
+		}
+
+		//exit
+		else if(key==vkEscape){
+			showExitMessageBox(lang.get("ExitGame?"), true);
+		}
+
+		//group
+		else if(key>='0' && key<'0'+Selection::maxGroups){
+			gui.groupKey(key-'0');
+		}
+
+		//hotkeys
+		if(gameCamera.getState()==GameCamera::sGame){
+			gui.hotKey(key);
+		}
+		else{
+			//rotate camera leftt
+			if(key=='A'){
+				gameCamera.setRotate(-1);
+			}
+
+			//rotate camera right
+			else if(key=='D'){
+				gameCamera.setRotate(1);
+			}
+
+			//camera up
+			else if(key=='S'){
+				gameCamera.setMoveY(1);
+			}
+
+			//camera down
+			else if(key=='W'){
+				gameCamera.setMoveY(-1);
+			}
 		}
 	}
 }
 
 void Game::keyUp(char key){
 
-	switch(key){
-	case 'A':
-	case 'D':
-		gameCamera.setRotate(0);
-		break;
+	if(!chatManager.getEditEnabled()){
+		switch(key){
+		case 'N':
+			renderNetworkStatus= false;
+			break;
+		case 'A':
+		case 'D':
+			gameCamera.setRotate(0);
+			break;
 
-	case 'W':
-	case 'S':
-		gameCamera.stopMoveY();
-		break;
+		case 'W':
+		case 'S':
+			gameCamera.stopMoveY();
+			break;
 
-	case vkUp:
-	case vkDown:
-		gameCamera.stopMoveZ();
-		break;
+		case vkUp:
+		case vkDown:
+			gameCamera.stopMoveZ();
+			break;
 
-	case vkLeft:
-	case vkRight:
-		gameCamera.stopMoveX();
-		break;
+		case vkLeft:
+		case vkRight:
+			gameCamera.stopMoveX();
+			break;
+		}
 	}
 }
 
+void Game::keyPress(char c){
+	chatManager.keyPress(c);
+}
+
 // ==================== PRIVATE ==================== 
 
 // ==================== render ==================== 
@@ -567,7 +600,6 @@ void Game::render2d(){
 	renderer.renderDisplay();
 	
 	//minimap
-
 	if(!config.getBool("PhotoMode")){
         renderer.renderMinimap();
 	}
@@ -580,11 +612,14 @@ void Game::render2d(){
 		renderer.renderMessageBox(exitMessageBox);
 	}
 
+	renderer.renderChatManager(&chatManager);
+
     //debug info
 	if(config.getBool("DebugMode")){
         string str;
 
         str+= "MouseXY: " + intToStr(mouseX) + "," + intToStr(mouseY)+"\n";
+		str+= "PosObjWord: " + intToStr(gui.getPosObjWorld().x) + "," + intToStr(gui.getPosObjWorld().y)+"\n";
         str+= "Render FPS: "+intToStr(lastRenderFps)+"\n";
         str+= "Update FPS: "+intToStr(lastUpdateFps)+"\n";
         str+= "GameCamera pos: "+floatToStr(gameCamera.getPos().x)+","+floatToStr(gameCamera.getPos().y)+","+floatToStr(gameCamera.getPos().z)+"\n";
@@ -593,6 +628,7 @@ void Game::render2d(){
 		str+= "Vertex count: "+intToStr(renderer.getPointCount())+"\n";
 		str+= "Frame count:"+intToStr(world.getFrameCount())+"\n";
 
+
         for(int i=0; i<world.getFactionCount(); ++i){
             str+= "Player "+intToStr(i)+" res: ";
             for(int j=0; j<world.getTechTree()->getResourceTypeCount(); ++j){
@@ -607,10 +643,18 @@ void Game::render2d(){
 			gui.getDisplay()->getColor(), 10, 500, false);
 	}
 
+	//network status
+	if(renderNetworkStatus){
+		Renderer::getInstance().renderText(
+			NetworkManager::getInstance().getGameNetworkInterface()->getNetworkStatus(), 
+			coreData.getMenuFontNormal(),
+			gui.getDisplay()->getColor(), 20, 500, false);
+	}
+
     //resource info
 	if(!config.getBool("PhotoMode")){
         renderer.renderResourceStatus();
-		renderer.renderConsole(console);
+		renderer.renderConsole(&console);
     }
 	
     //2d mouse
