@@ -245,7 +245,7 @@ void Renderer::initGame(Game *game){
 		else{
 			
 			//projected
-			glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE4, 
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, 
 				shadowTextureSize, shadowTextureSize, 
 				0, GL_LUMINANCE, GL_UNSIGNED_BYTE, NULL);
 		}
@@ -537,6 +537,7 @@ void Renderer::renderMouse3d(){
 		glDisable(GL_STENCIL_TEST);
 		glDepthFunc(GL_LESS);
 		glEnable(GL_COLOR_MATERIAL);
+		glDepthMask(GL_FALSE);
 
 		Vec2i pos= gui->getPosObjWorld();    
 		Vec3f pos3f= Vec3f(pos.x, map->getCell(pos)->getHeight(), pos.y); 
@@ -548,8 +549,15 @@ void Renderer::renderMouse3d(){
 			//selection building emplacement
 			float offset= building->getSize()/2.f-0.5f;
 			glTranslatef(pos3f.x+offset, pos3f.y, pos3f.z+offset);
-			color= Vec4f(1.f, 1.f, 1.f, 0.5f);
-			//modelRenderer->setCustomTexture(CoreData::getInstance().getCustomTexture());
+
+			//choose color
+			if(map->isFreeCells(pos, building->getSize(), fLand)){
+				color= Vec4f(1.f, 1.f, 1.f, 0.5f);
+			} 
+			else{
+				color= Vec4f(1.f, 0.f, 0.f, 0.5f);
+			}
+
 			modelRenderer->begin(true, true, false);
 			glColor4fv(color.ptr());
 			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, color.ptr());
@@ -623,24 +631,43 @@ void Renderer::renderTextureQuad(int x, int y, int w, int h, const Texture2D *te
 	assertGl();
 }
 
-void Renderer::renderConsole(const Console &console){
+void Renderer::renderConsole(const Console *console){
 	const Gui *gui= game->getGui();
-	
-	Console::LineIterator it= console.getLineBegin();
 
 	glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
 	glEnable(GL_BLEND);
 	glColor3fv(gui->getDisplay()->getColor().ptr());
 
 	textRenderer->begin(CoreData::getInstance().getConsoleFont());
-	for(int i=0; it!=console.getLineEnd(); it++, ++i){
-		textRenderer->render(it->c_str(), 10, i*20+10);
+	for(int i=0; i<console->getLineCount(); ++i){
+		textRenderer->render(console->getLine(i), 20, i*20+20);
 	}
 	textRenderer->end();
 
 	glPopAttrib();
 }
 
+void Renderer::renderChatManager(const ChatManager *chatManager){
+	Lang &lang= Lang::getInstance();
+
+	if(chatManager->getEditEnabled()){
+		string text;
+
+		if(chatManager->getTeamMode()){
+			text+= lang.get("Team");
+		}
+		else
+		{
+			text+= lang.get("All");
+		}
+		text+= ": " + chatManager->getText() + "_";
+	
+		textRenderer->begin(CoreData::getInstance().getConsoleFont());
+		textRenderer->render(text, 300, 150);
+		textRenderer->end();
+	}
+}
+
 void Renderer::renderResourceStatus(){
 	
 	const Metrics &metrics= Metrics::getInstance();
@@ -925,7 +952,6 @@ void Renderer::renderSurface(){
 	int lastTex=-1;
 	int currTex;
 	const World *world= game->getWorld();
-	const int thisTeamIndex= world->getThisTeamIndex();
 	const Map *map= world->getMap();
 	const Rect2i mapBounds(0, 0, map->getSurfaceW()-1, map->getSurfaceH()-1);
 	float coordStep= world->getTileset()->getSurfaceAtlas()->getCoordStep();
@@ -938,6 +964,7 @@ void Renderer::renderSurface(){
 
 	glEnable(GL_BLEND);
 	glEnable(GL_COLOR_MATERIAL);
+	glDisable(GL_ALPHA_TEST);
 
 	//fog of war tex unit
 	glActiveTexture(fowTexUnit);
@@ -962,7 +989,6 @@ void Renderer::renderSurface(){
 	glActiveTexture(baseTexUnit);
 
 	Quad2i scaledQuad= visibleQuad/Map::cellScale;
-	
 
 	PosQuadIterator pqi(map, scaledQuad);
 	while(pqi.next()){
@@ -976,50 +1002,42 @@ void Renderer::renderSurface(){
 			SurfaceCell *tc01= map->getSurfaceCell(pos.x, pos.y+1);
 			SurfaceCell *tc11= map->getSurfaceCell(pos.x+1, pos.y+1);
 	
-			if(
-				tc00->isExplored(thisTeamIndex) || 
-				tc10->isExplored(thisTeamIndex) || 
-				tc01->isExplored(thisTeamIndex) || 
-				tc11->isExplored(thisTeamIndex))
-			{
-				triangleCount+= 2;
-				pointCount+= 4;
+			triangleCount+= 2;
+			pointCount+= 4;
 
-				//set texture
-				currTex= static_cast<const Texture2DGl*>(tc00->getSurfaceTexture())->getHandle();
-				if(currTex!=lastTex){
-					lastTex=currTex;
-					glBindTexture(GL_TEXTURE_2D, lastTex);
-				}
-				
-
-				Vec2f surfCoord= tc00->getSurfTexCoord();
-
-				glBegin(GL_TRIANGLE_STRIP);
-	
-				//draw quad using immediate mode
-				glMultiTexCoord2fv(fowTexUnit, tc01->getFowTexCoord().ptr());
-				glMultiTexCoord2f(baseTexUnit, surfCoord.x, surfCoord.y+coordStep);
-				glNormal3fv(tc01->getNormal().ptr());
-				glVertex3fv(tc01->getVertex().ptr());
-
-				glMultiTexCoord2fv(fowTexUnit, tc00->getFowTexCoord().ptr());
-				glMultiTexCoord2f(baseTexUnit, surfCoord.x, surfCoord.y);
-				glNormal3fv(tc00->getNormal().ptr());
-				glVertex3fv(tc00->getVertex().ptr());
-
-				glMultiTexCoord2fv(fowTexUnit, tc11->getFowTexCoord().ptr());
-				glMultiTexCoord2f(baseTexUnit, surfCoord.x+coordStep, surfCoord.y+coordStep);
-				glNormal3fv(tc11->getNormal().ptr());
-				glVertex3fv(tc11->getVertex().ptr());
-
-				glMultiTexCoord2fv(fowTexUnit, tc10->getFowTexCoord().ptr());
-				glMultiTexCoord2f(baseTexUnit, surfCoord.x+coordStep, surfCoord.y);
-				glNormal3fv(tc10->getNormal().ptr());
-				glVertex3fv(tc10->getVertex().ptr());
-
-				glEnd();
+			//set texture
+			currTex= static_cast<const Texture2DGl*>(tc00->getSurfaceTexture())->getHandle();
+			if(currTex!=lastTex){
+				lastTex=currTex;
+				glBindTexture(GL_TEXTURE_2D, lastTex);
 			}
+
+			Vec2f surfCoord= tc00->getSurfTexCoord();
+
+			glBegin(GL_TRIANGLE_STRIP);
+
+			//draw quad using immediate mode
+			glMultiTexCoord2fv(fowTexUnit, tc01->getFowTexCoord().ptr());
+			glMultiTexCoord2f(baseTexUnit, surfCoord.x, surfCoord.y+coordStep);
+			glNormal3fv(tc01->getNormal().ptr());
+			glVertex3fv(tc01->getVertex().ptr());
+
+			glMultiTexCoord2fv(fowTexUnit, tc00->getFowTexCoord().ptr());
+			glMultiTexCoord2f(baseTexUnit, surfCoord.x, surfCoord.y);
+			glNormal3fv(tc00->getNormal().ptr());
+			glVertex3fv(tc00->getVertex().ptr());
+
+			glMultiTexCoord2fv(fowTexUnit, tc11->getFowTexCoord().ptr());
+			glMultiTexCoord2f(baseTexUnit, surfCoord.x+coordStep, surfCoord.y+coordStep);
+			glNormal3fv(tc11->getNormal().ptr());
+			glVertex3fv(tc11->getVertex().ptr());
+
+			glMultiTexCoord2fv(fowTexUnit, tc10->getFowTexCoord().ptr());
+			glMultiTexCoord2f(baseTexUnit, surfCoord.x+coordStep, surfCoord.y);
+			glNormal3fv(tc10->getNormal().ptr());
+			glVertex3fv(tc10->getVertex().ptr());
+
+			glEnd();
 		}
 	}
 	glEnd();
@@ -1779,7 +1797,7 @@ void Renderer::renderMenuBackground(const MenuBackground *menuBackground){
 				glPushMatrix();
 				
 				glColor4f(1.f, 1.f, 1.f, alpha);
-				glTranslatef(pos.x, waterHeight+0.001f, pos.y);
+				glTranslatef(pos.x, waterHeight+0.01f, pos.y);
 
 				glBegin(GL_TRIANGLE_STRIP);
 					glTexCoord2f(0.f, 1.f);
@@ -1807,45 +1825,40 @@ void Renderer::renderMenuBackground(const MenuBackground *menuBackground){
 bool Renderer::computePosition(const Vec2i &screenPos, Vec2i &worldPos){
     
 	assertGl();
-	
-	GLuint selectBuffer[Gui::maxSelBuff];
+	const Map* map= game->getWorld()->getMap();
 	const Metrics &metrics= Metrics::getInstance();
+	float depth= 0.0f;
+	GLdouble modelviewMatrix[16]; 
+	GLdouble projectionMatrix[16];
+	GLint viewport[4]= {0, 0, metrics.getScreenW(), metrics.getScreenH()};
+	GLdouble worldX;
+	GLdouble worldY;
+	GLdouble worldZ;
+	GLint screenX= (screenPos.x * metrics.getScreenW() / metrics.getVirtualW());
+	GLint screenY= (screenPos.y * metrics.getScreenH() / metrics.getVirtualH());	
 
-    glSelectBuffer(Gui::maxSelBuff, selectBuffer);
-    glMatrixMode(GL_PROJECTION);
-    glPushMatrix();
+	//get the depth in the cursor pixel
+	glReadPixels(screenX, screenY, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &depth);
 
-	GLint view[]= {0, 0, metrics.getVirtualW(), metrics.getVirtualH()};
-    glRenderMode(GL_SELECT);
-    glLoadIdentity();
-	gluPickMatrix(screenPos.x, screenPos.y, 1.f, 1.f, view);
-	gluPerspective(perspFov, metrics.getAspectRatio(), perspNearPlane, perspFarPlane);
+	//load matrices
+	loadProjectionMatrix();
     loadGameCameraMatrix();
 
-	//compute pos 1
-    renderSurfaceFast();
-	bool validPos= false;
-	if(glRenderMode(GL_RENDER)>0){
-		glRenderMode(GL_SELECT);
-		Vec2i pos= Vec2i(selectBuffer[4], selectBuffer[3]);
-		renderTile(Vec2i(selectBuffer[4], selectBuffer[3]));
+	//get matrices
+	glGetDoublev(GL_MODELVIEW_MATRIX, modelviewMatrix);
+	glGetDoublev(GL_PROJECTION_MATRIX, projectionMatrix);
 
-		//compute pos 2
-		if(glRenderMode(GL_RENDER)>0){
-			worldPos= Vec2i(selectBuffer[4], selectBuffer[3]);
-			validPos= true;
-		}
-		else{
-			worldPos= pos*Map::cellScale;
-			validPos= true;
-		}
-	}
-	glMatrixMode(GL_PROJECTION);
-	glPopMatrix();
+	//get the world coordinates
+	gluUnProject(
+		screenX, screenY, depth, 
+		modelviewMatrix, projectionMatrix, viewport,
+		&worldX, &worldY, &worldZ);
 
-	assertGl();
-
-	return validPos;
+	//conver coords to int
+	worldPos= Vec2i(static_cast<int>(worldX+0.5f), static_cast<int>(worldZ+0.5f));
+	
+	//clamp coords to map size
+	return map->isInside(worldPos);
 }
 
 void Renderer::computeSelected(Selection::UnitContainer &units, const Vec2i &posDown, const Vec2i &posUp){
@@ -2093,8 +2106,8 @@ string Renderer::getGlMoreInfo(){
 void Renderer::autoConfig(){
 	
 	Config &config= Config::getInstance();
-	bool nvidiaCard = toLower(getGlVendor()).find("nvidia")!=string::npos;
-	bool atiCard = toLower(getGlVendor()).find("ati")!=string::npos;
+	bool nvidiaCard= toLower(getGlVendor()).find("nvidia")!=string::npos;
+	bool atiCard= toLower(getGlVendor()).find("ati")!=string::npos;
 	bool shadowExtensions = isGlExtensionSupported("GL_ARB_shadow") && isGlExtensionSupported("GL_ARB_shadow_ambient");
 		
 	//3D textures
@@ -2232,48 +2245,6 @@ Vec4f Renderer::computeWaterColor(float waterLevel, float cellHeight){
 
 // ==================== fast render ==================== 
 
-//this fc renders surface for selection
-void Renderer::renderSurfaceFast(){
-	
-	const World *world= game->getWorld();
-	const Map *map= world->getMap();
-	const Rect2i mapBounds(0, 0, map->getSurfaceW()-1, map->getSurfaceH()-1);
-
-	glMatrixMode(GL_MODELVIEW);
-	glPushMatrix();
-
-	glPushAttrib(GL_ENABLE_BIT);
-	glDisable(GL_LIGHTING);
-	glDisable(GL_TEXTURE_2D);
-
-	glTranslatef(-0.5f, 0.f, -0.5f);
-	glInitNames();
-	
-	Quad2i scaledQuad= visibleQuad/Map::cellScale;
-	
-	PosQuadIterator pqi(map, scaledQuad);
-	while(pqi.next()){
-		const Vec2i &pos= pqi.getPos();
-		if(mapBounds.isInside(pos)){
-			glPushName(pos.y);
-			glPushName(pos.x);
-			glBegin(GL_TRIANGLE_FAN);
-             	glVertex3fv(map->getSurfaceCell(pos.x, pos.y+1)->getVertex().ptr());
-             	glVertex3fv(map->getSurfaceCell(pos.x, pos.y)->getVertex().ptr());
-              	glVertex3fv(map->getSurfaceCell(pos.x+1 ,pos.y)->getVertex().ptr());
-              	glVertex3fv(map->getSurfaceCell(pos.x+1, pos.y+1)->getVertex().ptr());
-			glEnd();
-			glPopName();
-			glPopName();
-		}
-	}
-
-	//restore
-	glPopAttrib();
-
-	glPopMatrix();
-}
-
 //render units for selection purposes
 void Renderer::renderUnitsFast(){
 	const World *world= game->getWorld();
@@ -2328,13 +2299,24 @@ void Renderer::renderObjectsFast(){
 
     assertGl();
 
-	glPushAttrib(GL_ENABLE_BIT);
-	glDisable(GL_TEXTURE_2D);
+	glPushAttrib(GL_ENABLE_BIT| GL_TEXTURE_BIT);
 	glDisable(GL_LIGHTING);
 
     glAlphaFunc(GL_GREATER, 0.5f);
 
-	modelRenderer->begin(false, false, false);
+	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
+		
+	//set color to the texture alpha		
+	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_REPLACE);
+	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PRIMARY_COLOR);
+	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB, GL_SRC_COLOR);
+	
+	//set alpha to the texture alpha
+	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_REPLACE);
+	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_TEXTURE);
+	glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA, GL_SRC_ALPHA);
+
+	modelRenderer->begin(false, true, false);
 	int thisTeamIndex= world->getThisTeamIndex();
 
 	PosQuadIterator pqi(map, visibleQuad, Map::cellScale);
@@ -2432,8 +2414,6 @@ void Renderer::init3dList(){
 		glClearColor(fowColor.x, fowColor.y, fowColor.z, fowColor.w);			
 		glFrontFace(GL_CW);											
 		glEnable(GL_CULL_FACE);											
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
 		loadProjectionMatrix();
 
 		//texture state
@@ -2618,6 +2598,8 @@ void Renderer::loadProjectionMatrix(){
 
 	clipping= photoMode ? perspFarPlane*100 : perspFarPlane; 
  	
+	glMatrixMode(GL_PROJECTION);
+	glLoadIdentity();
 	gluPerspective(perspFov, metrics.getAspectRatio(), perspNearPlane, clipping);
 
     assertGl();
