@@ -236,8 +236,6 @@ void Gui::mouseUpLeftGraphics(int x, int y){
 
 void Gui::mouseMoveGraphics(int x, int y){
    
-	Vec2i posObjScreen= Vec2i(x,y);
-
 	//compute selection
 	if(selectionQuad.isEnabled()){
 		selectionQuad.setPosUp(Vec2i(x, y));
@@ -249,7 +247,7 @@ void Gui::mouseMoveGraphics(int x, int y){
 
 	//compute position for building
 	if(isPlacingBuilding()){
-		validPosObjWorld= Renderer::getInstance().computePosition(posObjScreen, posObjWorld);
+		validPosObjWorld= Renderer::getInstance().computePosition(Vec2i(x,y), posObjWorld);
 	}
 
 	display.setInfoText("");
@@ -462,55 +460,57 @@ void Gui::clickCommonCommand(CommandClass commandClass){
 }
 
 void Gui::mouseDownDisplayUnitSkills(int posDisplay){
-	if(posDisplay != cancelPos){
-		if(posDisplay!=meetingPointPos){
-			const Unit *unit= selection.getFrontUnit();	
-			
-			//uniform selection
-			if(selection.isUniform()){
-				if(unit->getFaction()->reqsOk(display.getCommandType(posDisplay))){
-					activeCommandType= display.getCommandType(posDisplay);
-					activeCommandClass= activeCommandType->getClass();
+	if(!selection.isEmpty()){
+		if(posDisplay != cancelPos){
+			if(posDisplay!=meetingPointPos){
+				const Unit *unit= selection.getFrontUnit();	
+				
+				//uniform selection
+				if(selection.isUniform()){
+					if(unit->getFaction()->reqsOk(display.getCommandType(posDisplay))){
+						activeCommandType= display.getCommandType(posDisplay);
+						activeCommandClass= activeCommandType->getClass();
+					}
+					else{
+						posDisplay= invalidPos;
+						activeCommandType= NULL;
+						activeCommandClass= ccStop;
+						return;
+					}
 				}
+
+				//non uniform selection
 				else{
-					posDisplay= invalidPos;
 					activeCommandType= NULL;
-					activeCommandClass= ccStop;
-					return;
+					activeCommandClass= display.getCommandClass(posDisplay);
+				}
+
+				//give orders depending on command type
+				if(!selection.isEmpty()){
+					const CommandType *ct= selection.getUnit(0)->getType()->getFirstCtOfClass(activeCommandClass);
+					if(activeCommandType!=NULL && activeCommandType->getClass()==ccBuild){
+						assert(selection.isUniform());
+						selectingBuilding= true;
+					}
+					else if(ct->getClicks()==cOne){
+						invalidatePosObjWorld();
+						giveOneClickOrders();
+					}
+					else{
+						selectingPos= true;
+						activePos= posDisplay;
+					}
 				}
 			}
-
-			//non uniform selection
 			else{
-				activeCommandType= NULL;
-				activeCommandClass= display.getCommandClass(posDisplay);
-			}
-
-			//give orders depending on command type
-			if(!selection.isEmpty()){
-				const CommandType *ct= selection.getUnit(0)->getType()->getFirstCtOfClass(activeCommandClass);
-				if(activeCommandType!=NULL && activeCommandType->getClass()==ccBuild){
-					assert(selection.isUniform());
-					selectingBuilding= true;
-				}
-				else if(ct->getClicks()==cOne){
-					invalidatePosObjWorld();
-					giveOneClickOrders();
-				}
-				else{
-					selectingPos= true;
-					activePos= posDisplay;
-				}
+				activePos= posDisplay;
+				selectingMeetingPoint= true;
 			}
 		}
 		else{
-			activePos= posDisplay;
-			selectingMeetingPoint= true;
+			commander->tryCancelCommand(&selection);
 		}
 	}
-	else{
-		commander->tryCancelCommand(&selection);
-	}
 }
 
 void Gui::mouseDownDisplayUnitBuild(int posDisplay){
@@ -739,7 +739,6 @@ void Gui::addOrdersResultToConsole(CommandClass cc, CommandResult result){
         
     switch(result){
 	case crSuccess: 
-        console->addStdMessage("Ok");
 		break;
 	case crFailReqs: 
         switch(cc){
@@ -849,7 +848,6 @@ bool Gui::computeTarget(const Vec2i &screenPos, Vec2i &targetPos, const Unit *&t
 			return false;
 		}
 	}
-
 }
 
 }}//end namespace
