@@ -14,14 +14,14 @@
 #include <SDL.h>
 #include <iostream>
 
-#define MAIN_FUNCTION(X) int main(int, char**)                               \
+#define MAIN_FUNCTION(X) int main(int argc, char **argv)                     \
 {                                                                            \
     if(SDL_Init(SDL_INIT_EVERYTHING) < 0)  {                                 \
         std::cerr << "Couldn't initialize SDL: " << SDL_GetError() << "\n";  \
         return 1;                                                            \
     }                                                                        \
 	SDL_EnableUNICODE(1);													 \
-    int result = X();                                                        \
+    int result = X(argc, argv);                                              \
     SDL_Quit();                                                              \
     return result;                                                           \
 }
