@@ -14,6 +14,6 @@
 
 #include <windows.h>
 
-#define MAIN_FUNCTION(X) int __stdcall WinMain(HINSTANCE, HINSTANCE, LPSTR, int){return X();}
+#define MAIN_FUNCTION(X) int main(int argc, char *argv[]){return X(argc, argv);}
 
-#endif
+#endif
\ No newline at end of file
